/*
 * kernel_arch/i386/kernel/timer.c
 *
 * Copyright (c) 2003-2004 Lukasz Dembinski <dembol@linux.net>
 * All Rights Reserved
 * 
 * Date:	2004/01
 * Author: 	Lukasz Dembinski
 * Info:	timer.c core file
 * Contact:	mailto: <dembol@linux.net>
 *
 */

#include <agnix/agnix.h>
#include <agnix/irq.h>
#include <agnix/tasks.h>
#include <agnix/timer.h>
#include <agnix/memory.h>
#include <agnix/adi/adi.h>
#include <agnix/spinlock.h>
#include <agnix/timers.h>
#include <agnix/math64.h>

#define TIMER_INC_VAL	(1e6 / TIMER_HZ)	/* usec */
#define USEC_IN_SEC	1000000

struct timeval_s current_time;
extern struct chip_s rtc_chip;
u32 jiffies_64[2] = { 0, 0 };

void timer_usec_overflow(void)
{
    do {
	current_time.tv_usec -= USEC_IN_SEC;
	current_time.tv_sec++;
    } while (current_time.tv_usec > USEC_IN_SEC);
}

void timer_actualize(void)
{
    current_time.tv_usec += TIMER_INC_VAL;
    
    if (current_time.tv_usec > USEC_IN_SEC)
	timer_usec_overflow();
}

void jiffies_actualize(void)
{
    add_64_32(jiffies_64, jiffies_64, 1);
}

void timer_irq(u32 i)
{
    if (--current_task->t_count <= 0) {
	current_task->t_state |= TASK_STAT_RESCHED;
    }
    else
	current_task->t_state &= ~TASK_STAT_RESCHED;
    
    timer_actualize();
    jiffies_actualize();
    run_timers();
}

struct irq_routine_s irq_timer = { timer_irq, 0, 0, };

int __init timer_init(void)
{
    SPIN_LOCK;

    rtc_ops(&rtc_chip)->gettime(&rtc_chip, &current_time);
    install_irq(0, &irq_timer);

    SPIN_UNLOCK;

    return 0;
}

int timer_gettimeofday(struct timeval_s *tv)
{
    memcpy(tv, &current_time, sizeof(struct timeval_s));

    return 0;
}
