/*
 * kernel_super/memory/alloc.c
 *
 * Copyright (c) 2003-2004 Lukasz Dembinski <dembol@linux.net>
 * All Rights Reserved
 * 
 * Date:	2004/01
 * Author: 	Lukasz Dembinski
 * Info:	alloc.c core file
 * Contact:	mailto: <dembol@linux.net>
 *
 */
 
#include <agnix/init.h>
#include <asm/types.h>
#include <asm/bitops.h>
#include <asm/memory.h>
#include <asm/paging.h>

#include <agnix/memory.h>
#include <agnix/bugs.h>
#include <agnix/console.h>

#define MOD_NAME	"MEM: "
#define NO_PAGE		0

struct allocator_s mem_alloc;
extern char _end;
extern struct main_memory_s main_memory;

extern int bootmem_init_bh(void);
extern int bootmem_init_dh(void);

//int printk_mem = 0;

void _reserve_pages(struct allocator_s *allocator, u32 start_page, u32 size)
{
    u32 idx_start = start_page - allocator->mem_start;
    u32 idx_end = idx_start + size;
    u32 i;
    
    for (i = idx_start; i < idx_end; i++) {
	if (test_and_set_bit((int)i, (void *)allocator->mem_table)) {
	    BUG(MOD_NAME, BUG_RESERVE_PAGES);
	    return;
	}
    }
}

void _free_pages(struct allocator_s *allocator, u32 start_page, u32 size)
{
    u32 idx_start = start_page - allocator->mem_start;
    u32 idx_end = idx_start + size;
    u32 i;
    
    for (i = idx_start; i < idx_end; i++) {
	if (!test_and_clear_bit((int)i, (void *)allocator->mem_table)) {
	    BUG(MOD_NAME, BUG_FREE_PAGES);
	    return;
	}
    }
}

u32 _get_free_pages(struct allocator_s *allocator, u32 size)
{
    u32 i = 0;
    u32 j = 0;

    for (i = 0; i < allocator->mem_table_size; i++) {
	if (test_bit((int)i, (void *)allocator->mem_table))
	    continue;
	
	for (j = 0; j < size; j++) {
	    if (test_bit((int)(i + j), (void *)allocator->mem_table))
		goto bad_area;
	}
	
	return (allocator->mem_start + i);
	
bad_area:
	i += j;
    }
    
    return NO_PAGE;
}

void put_free_pages(u32 addr, u8 order)
{
    _free_pages(&mem_alloc, (addr - PAGE_OFFSET) >> PAGE_SHIFT, (u32)(1 << order));    
}


u32 get_free_pages(u8 order)
{
    u32 page;
    page = _get_free_pages(&mem_alloc, (u32)(1 << order));
    
    if (page != NO_PAGE)
	_reserve_pages(&mem_alloc, page, (u32)(1 << order));

/*    if (printk_mem)
        printk("get_free_pages() = %x\n", (page << PAGE_SHIFT) + PAGE_OFFSET); */

    return ((page << PAGE_SHIFT) + PAGE_OFFSET);
}

void __init _memalloc_init(u32 *mem_table, u32 mem_start, u32 mem_end)
{
    memset(&mem_alloc, 0, sizeof(mem_alloc));
    
    mem_alloc.mem_table = mem_table;
    mem_alloc.mem_start = mem_start;
    mem_alloc.mem_end = mem_end;
    mem_alloc.mem_table_size = (mem_end - mem_start);

    memset(mem_table, 0, mem_alloc.mem_table_size);
    
/*    we cannot do it, because system console is not initialized yet */
/*    printk(MOD_NAME "initializing memory allocator\n"); */
}

int __init memalloc_init(void)
{
    bootmem_init_bh();

    _memalloc_init((u32 *)&_end, 0, main_memory.mem_size);
    _reserve_pages(&mem_alloc, 0, 2);

    bootmem_init_dh();
    
    return 0;
}
